package com.example.service;

import com.example.entity.IncomingShipmentDataEntity;
import com.example.entity.ItemEntity;
import com.example.entity.MonthlyStockEntity;
import com.example.repository.IncomingShipmentDataRepository;
import com.example.repository.ItemRepository;
import com.example.repository.MonthlyStockRepository;
import com.example.repository.StorageLocationRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * 在庫管理に関するビジネスロジックを提供するサービス
 */
@Service
public class InventoryService {

    @Autowired
    private ItemRepository itemRepository;

    @Autowired
    private StorageLocationRepository storageLocationRepository;

    @Autowired
    private IncomingShipmentDataRepository incomingShipmentDataRepository;

    @Autowired
    private MonthlyStockRepository monthlyStockRepository;

    /**
     * 品目コードと保管場所IDを条件に在庫情報を検索し、一覧表示するためのデータを取得する
     */
    public List<ItemEntity> searchInventory(String itemCode, Long storageLocationId) {
        // 品目テーブルを検索
        List<ItemEntity> items = itemRepository.findByItemCodeContaining(itemCode);

        // 現在日時を取得
        Date currentDate = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(currentDate);
        int currentDay = calendar.get(Calendar.DAY_OF_MONTH);

        // 月初日時と前日の日時を計算
        calendar.set(Calendar.DAY_OF_MONTH, 1);
        Date startOfMonth = calendar.getTime();
        calendar.add(Calendar.DATE, -1);
        Date endOfPreviousDay = calendar.getTime();

        // 当日の日時範囲を計算
        calendar.setTime(currentDate);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date startOfCurrentDay = calendar.getTime();
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        Date endOfCurrentDay = calendar.getTime();

        // 各品目について在庫情報を計算
        for (ItemEntity item : items) {
            Long itemId = item.getId();

            // 月初在庫数量を取得
            Integer beginningOfMonthInventory = monthlyStockRepository.findQuantityByItemIdAndStorageLocationId(itemId, storageLocationId);

            // 前日迄の入庫数と出庫数を取得
            Integer previousDayStockNumber = 0;
            Integer previousDayDispatchNumber = 0;
            if (currentDay != 1) {
                previousDayStockNumber = incomingShipmentDataRepository.findQuantityByItemIdAndStorageLocationIdAndTransactionTypeBetweenDates(itemId, storageLocationId, startOfMonth, endOfPreviousDay, IncomingShipmentDataEntity.TRANSACTION_TYPE_INCOMING);
                previousDayDispatchNumber = incomingShipmentDataRepository.findQuantityByItemIdAndStorageLocationIdAndTransactionTypeBetweenDates(itemId, storageLocationId, startOfMonth, endOfPreviousDay, IncomingShipmentDataEntity.TRANSACTION_TYPE_OUTGOING);
            }

            // 当日の入庫数と出庫数を取得
            Integer currentStockNumber = incomingShipmentDataRepository.findQuantityByItemIdAndStorageLocationIdAndTransactionTypeBetweenDates(itemId, storageLocationId, startOfCurrentDay, endOfCurrentDay, IncomingShipmentDataEntity.TRANSACTION_TYPE_INCOMING);
            Integer currentDispatchNumber = incomingShipmentDataRepository.findQuantityByItemIdAndStorageLocationIdAndTransactionTypeBetweenDates(itemId, storageLocationId, startOfCurrentDay, endOfCurrentDay, IncomingShipmentDataEntity.TRANSACTION_TYPE_OUTGOING);

            // 現在在庫数を計算
            Integer currentInventory = (beginningOfMonthInventory != null ? beginningOfMonthInventory : 0)
                    + (previousDayStockNumber != null ? previousDayStockNumber : 0)
                    + (currentStockNumber != null ? currentStockNumber : 0)
                    - (previousDayDispatchNumber != null ? previousDayDispatchNumber : 0)
                    - (currentDispatchNumber != null ? currentDispatchNumber : 0);

            // TODO: 計算した在庫情報をItemEntityにセットするロジックを実装する
        }

        return items;
    }
}