from django.views import View
from django.shortcuts import render
from .models import Item, IncomingShipmentData, StorageLocation, MonthlyStock
from django.db.models import Sum, Q
from datetime import datetime

class ItemInventoryListView(View):
    # 品目在庫管理一覧画面のビュー
    template_name = 'item_inventory_list.html'

    def get(self, request, *args, **kwargs):
        # １．初期処理
        # 1.1. 権限チェック（実装は省略）
        # 1.2. 保管場所の取得
        storage_locations = StorageLocation.objects.all().values('id', 'storage_location_name')
        context = {
            'storage_locations': storage_locations,
        }
        return render(request, self.template_name, context)

    def post(self, request, *args, **kwargs):
        # 2．検索ボタン
        item_code_search = request.POST.get('itemCodeSearch')
        storage_location_search = request.POST.get('storageLocationSearch')
        today = datetime.now()
        beginning_of_month = today.replace(day=1, hour=0, minute=0, second=0, microsecond=0)
        end_of_previous_day = today.replace(hour=23, minute=59, second=59, microsecond=999999) - timedelta(days=1)
        start_of_today = today.replace(hour=0, minute=0, second=0, microsecond=0)
        end_of_today = today.replace(hour=23, minute=59, second=59, microsecond=999999)

        # 2.1. 品目テーブルを検索
        items = Item.objects.filter(item_code__icontains=item_code_search) if item_code_search else Item.objects.all()

        # 2.2. 入出庫テーブルを取得
        inventory_list = []
        for item in items:
            # 2.2.1 月初在庫テーブルを検索
            beginning_inventory = MonthlyStock.objects.filter(
                item=item,
                storage_location_id=storage_location_search,
                month_of_collection=today.month
            ).aggregate(Sum('quantity'))['quantity__sum'] or 0

            # 2.2.1.1 前日迄入庫数・前日迄出庫数
            previous_incoming = IncomingShipmentData.objects.filter(
                item=item,
                transaction_location=storage_location_search,
                transaction_date__gte=beginning_of_month,
                transaction_date__lte=end_of_previous_day,
                transaction_type=1  # 入庫区分
            ).aggregate(Sum('transaction_quantity'))['transaction_quantity__sum'] or 0

            previous_outgoing = IncomingShipmentData.objects.filter(
                item=item,
                transaction_location=storage_location_search,
                transaction_date__gte=beginning_of_month,
                transaction_date__lte=end_of_previous_day,
                transaction_type=2  # 出庫区分
            ).aggregate(Sum('transaction_quantity'))['transaction_quantity__sum'] or 0

            # 2.2.1.3 当日入庫数
            current_incoming = IncomingShipmentData.objects.filter(
                item=item,
                transaction_location=storage_location_search,
                transaction_date__gte=start_of_today,
                transaction_date__lte=end_of_today,
                transaction_type=1  # 入庫区分
            ).aggregate(Sum('transaction_quantity'))['transaction_quantity__sum'] or 0

            # 2.2.1.4 当日出庫数
            current_outgoing = IncomingShipmentData.objects.filter(
                item=item,
                transaction_location=storage_location_search,
                transaction_date__gte=start_of_today,
                transaction_date__lte=end_of_today,
                transaction_type=2  # 出庫区分
            ).aggregate(Sum('transaction_quantity'))['transaction_quantity__sum'] or 0

            # 2.2.1.5 現在在庫数
            current_inventory = beginning_inventory + previous_incoming + current_incoming - previous_outgoing - current_outgoing

            inventory_list.append({
                'item_id': item.id,
                'item_code': item.item_code,
                'item_name': item.item_name,
                'item_type': item.item_category_id,
                'main_trading_partner': item.supplier_code.supplier_name if item.supplier_code else '',
                'beginning_of_month_inventory': beginning_inventory,
                'previous_day_stock_number': previous_incoming,
                'previous_day_dispatch_number': previous_outgoing,
                'current_stock_number': current_incoming,
                'current_dispatch_number': current_outgoing,
                'current_inventory': current_inventory,
            })

        # 2.3. 取得したデータを画面に返却
        context = {
            'inventory_list': inventory_list,
            'storage_locations': StorageLocation.objects.all().values('id', 'storage_location_name'),
        }
        return render(request, self.template_name, context)